/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.freemarkerdor;

import com.sun.jersey.api.client.Client;
import com.sun.jersey.api.client.ClientResponse;
import com.sun.jersey.api.client.WebResource;
import com.sun.jersey.core.header.MediaTypes;
import com.sun.jersey.core.util.MultivaluedMapImpl;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.util.Locale;
import java.util.concurrent.Callable;
import javax.ws.rs.core.Cookie;
import javax.ws.rs.core.MultivaluedMap;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class UITest extends Object {
    private static File dir;
    private static Callable<Void> stop;
    private WebResource webResource;

    public UITest() throws Exception {
    }

    @BeforeClass
    public static void localeEnglish() throws Exception {
        Locale.setDefault(Locale.ENGLISH);
        dir = File.createTempFile("quoridor", ".dir");
        dir.delete();
        dir.mkdirs();
        System.setProperty("quoridor.dir", dir.getPath());
        stop = UI.startServers(9991);

        File passwd = new File(dir, "passwd");
        FileOutputStream os = new FileOutputStream(passwd);
        os.write("test=pes\n".getBytes("UTF-8"));
        os.close();
    }

    @Before
    public void setUp() throws Exception {

        Client client = new Client();
        webResource = client.resource(new URI("http://localhost:9991/"));
    }

    @AfterClass
    public static void cleanUpAll() throws Exception {
        deleteRec(dir);
        if (stop != null) {
            stop.call();
        }
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }

    /**
     * Test if a WADL document is available at the relative path
     * "application.wadl".
     */
    @Test public void testApplicationWadl() {
        String serviceWadl = webResource.path("application.wadl").
                accept(MediaTypes.WADL).get(String.class);
        assertTrue(serviceWadl.length() > 0);
    }

    @Test public void testGetIndexPage() throws Exception {
        String res = webResource.accept("text/html").get(String.class);
        if (res.indexOf("Quoridor") == -1) {
            fail("Wrong index.html:\n" + res);
        }
        if (res.indexOf("Login") == -1) {
            fail("Wrong index.html:\n" + res);
        }
        if (res.indexOf("action=\"/login\"") == -1) {
            fail("Wrong index.html:\n" + res);
        }
        if (res.toLowerCase().indexOf("error") != -1) {
            fail("There was an error:\n" + res);
        }
        res = webResource.cookie(Cookie.valueOf("login=jarda")).accept("text/html").get(String.class);
        if (res.indexOf("action=\"/games/create\"") == -1) {
            fail(res);
        }
    }

    @Test public void testLogin() throws Exception {
        MultivaluedMap formData = new MultivaluedMapImpl();
        formData.add("name", "test");
        formData.add("password", "pes");
        ClientResponse response = webResource.path("login").
            accept("text/html").type("application/x-www-form-urlencoded").
            post(ClientResponse.class, formData);
        final String res = response.getEntity(String.class);
        assertEquals("OK", ClientResponse.Status.OK, response.getClientResponseStatus());
        if (res.indexOf("You are logged in as test") == -1) {
            fail("res: " + res);
        }
    }


    @Test public void testCreateGameWrongUsers() throws Exception {
        ClientResponse res = webResource.path("games/create").
            queryParam("white", "unknown1").
            queryParam("black", "unknown2").
            cookie(Cookie.valueOf("login=test")).
            accept("text/html").
            get(ClientResponse.class);
        final String txt = res.getEntity(String.class);
        assertEquals("Rejected, unknown user\n" + txt, ClientResponse.Status.NOT_FOUND, res.getClientResponseStatus());
        if (txt.indexOf("You (test) must be") == -1) {
            fail(txt);
        }
    }

    @Test public void testCreateGameOK() throws Exception {
        ClientResponse res = webResource.path("games/create").
            queryParam("white", "unknown1").
            queryParam("black", "test").
            cookie(Cookie.valueOf("login=test")).
            accept("text/html").
            get(ClientResponse.class);

        final String txt = res.getEntity(String.class);
        assertEquals("OK\n" + txt, ClientResponse.Status.OK, res.getClientResponseStatus());

        String[] games = new File(dir, "games").list();
        assertEquals("One game exists", 1, games.length);

        if (txt.indexOf(games[0]) == -1) {
            fail(games[0] + " expected inside of:\n" + txt);
        }
    }

}
